import pandas as pd

from sklearn.base import TransformerMixin

class DateFilter(TransformerMixin):

    def __init__(self, field_name, delta="30D"):
        self.field_name = field_name
        self.delta = pd.Timedelta(delta)


    def fit(self, X, y=None):
        # stateless transformer
        return self

    def transform(self, X):
        max_date = X[self.field_name].max() 
        # not in fit() method since we should not look 
        # exact date for test data we conceptually look
        # users last 14 days behaviour

        filtered_df = X[ max_date - self.delta < X[self.field_name]]
        return filtered_df



from sklearn.base import TransformerMixin

class ColumnExtractor(TransformerMixin):

    def __init__(self, cols):
        self.cols = cols

    def fit(self, X, y=None):
        # stateless transformer
        return self

    def transform(self, X):
        # assumes X is a DataFrame
        Xcols = X[self.cols]
        return Xcols



import pandas as pd

from sklearn.base import TransformerMixin

class DateTimeFeatureExtractor(TransformerMixin):

    def __init__(self, agg_by, date_field_name, freq="1D", limit="30D", rel_field_name=None, round_to="1D"):
        self.limit = limit
        self.rel_field_name = rel_field_name
        self.date_field_name = date_field_name
        self.agg_by = agg_by
        self.freq = freq
        self.round_to = round_to

    def fit(self, X, y=None):
        # stateless transformer
        return self

    def transform(self, X):
        max_date = X[self.date_field_name].max().round(self.round_to)
        dates = pd.date_range(max_date - pd.Timedelta(self.limit), max_date, freq=self.freq)[1:]
        column_names = [self.agg_by]
        for i, date in enumerate(dates):
            column_name =  "%s_%s_%s" % (self.freq, self.rel_field_name or "COUNT", str(i))
            column_names.append(column_name)
            X[column_name] =  (
                            (date - pd.Timedelta(self.freq) < X[self.date_field_name]) & 
                                     (X[self.date_field_name] <= date )
                        ) * (X[self.rel_field_name] if self.rel_field_name else 1)

        new_df = X.groupby([self.agg_by]).sum()

        new_df.reset_index(inplace=True)

        return new_df[column_names] 



import pandas as pd

from sklearn.base import TransformerMixin


class Converter(TransformerMixin):

    def __init__(self, _type):
        self._type = _type

    def fit(self, X, y=None):
        # stateless transformer
        return self

    def transform(self, X):
        # assumes X is a DataFrame
        Xcols = X.astype(self._type)
        return Xcols

class DateConverter(TransformerMixin):

    def __init__(self, field_name, format):
        self.format = format
        self.field_name = field_name

    def fit(self, X, y=None, **fit_params):
        return self

    def transform(self, X, **transform_params):
        cleaned = X
        cleaned[self.field_name] = pd.to_datetime(X[self.field_name], format=self.format)
        return cleaned



from sklearn.base import TransformerMixin


class StringLengthFilter(TransformerMixin):

    def __init__(self, field_name, length):
        self.field_name = field_name
        self.length = length

    def fit(self, X, y=None):
        # stateless transformer
        return self

    def transform(self, X):
        # assumes X is a DataFrame
        Xcols = X[X[self.field_name].str.len() > self.length]
        return Xcols


from sklearn.base import TransformerMixin

class SringReplacer(TransformerMixin):
   
    def __init__(self, _from, to):
        self._from = _from
        self.to = to


    def fit(self, X, y=None, **fit_params):
        return self

    def transform(self, X, **transform_params):
        cleaned = X.applymap(lambda x: str(x).replace(self._from, self.to))
        return cleaned
